package com.tenlionsoft.aimz_k

import com.google.gson.Gson
import com.tenlionsoft.aimz_k.model.CoverSealedBean
import com.tenlionsoft.aimz_k.model.MsgBean
import com.tenlionsoft.aimz_k.model.MsgCategoryBean
import com.tenlionsoft.aimz_k.model.MsgConvertBean
import com.tenlionsoft.aimz_k.model.MsgTypeStateEnum
import com.tenlionsoft.baselib.contacts.ProjectConfig

object ConvertBeanUtils {

    fun covertBean(str: String): CoverSealedBean {
        val gson = Gson()
        val sBean = gson.fromJson(str, MsgConvertBean::class.java)
        var type: Int = -1
        if (sBean.messageType != ProjectConfig.MSG_STATUS) {
            when (sBean.messageType) {
                ProjectConfig.MSG_TEXT -> type = MsgTypeStateEnum.MSG_FROM_OTHER_TXT
                ProjectConfig.MSG_IMG -> type = MsgTypeStateEnum.MSG_FROM_OTHER_IMG
                ProjectConfig.MSG_FILE -> type = MsgTypeStateEnum.MSG_FROM_OTHER_FILE
                ProjectConfig.MSG_VIDEO -> type = MsgTypeStateEnum.MSG_FROM_OTHER_MOVIE
            }
            return CoverSealedBean.Msg(
                MsgBean(
                    messageId = sBean.messageId,
                    messageType = sBean.messageType,
                    metadata = sBean.metadata,
                    receiverId = sBean.receiver.receiverId,
                    receiverType = sBean.receiver.receiverType,
                    senderType = sBean.sender.senderType,
                    senderId = sBean.sender.senderId,
                    body = sBean.body,
                    msgType = type,
                    timestamp = sBean.timestamp,
                    status = sBean.status,
                    customMessageType = ""
                )
            )
        } else {
            return CoverSealedBean.StateBean(sBean)
        }
    }

    fun convertBeanToMsgBean(cBean: MsgConvertBean): MsgBean {
        var type: Int = -1
        when (cBean.messageType) {
            ProjectConfig.MSG_TEXT -> type = MsgTypeStateEnum.MSG_TO_OTHER_TXT
            ProjectConfig.MSG_IMG -> type = MsgTypeStateEnum.MSG_TO_OTHER_IMG
            ProjectConfig.MSG_FILE -> type = MsgTypeStateEnum.MSG_TO_OTHER_FILE
            ProjectConfig.MSG_VIDEO -> type = MsgTypeStateEnum.MSG_TO_OTHER_MOVIE
        }
        return MsgBean(
            messageId = cBean.messageId,
            messageType = cBean.messageType,
            metadata = cBean.metadata,
            receiverId = cBean.receiver.receiverId,
            receiverType = cBean.receiver.receiverType,
            senderType = cBean.sender.senderType,
            senderId = cBean.sender.senderId,
            body = cBean.body,
            msgType = type,
            timestamp = cBean.timestamp,
            status = cBean.status,
            customMessageType = ""
        )
    }

    fun convertCategoryBean(msgBean: MsgBean): MsgCategoryBean {
        return MsgCategoryBean(
            avatar = "",
            messageType = msgBean.messageType,
            messageId = msgBean.messageId,
            receiverType = msgBean.receiverType,
            receiverId = msgBean.receiverId,
            senderType = msgBean.senderType,
            senderId = msgBean.senderId,
            status = msgBean.status,
            timestamp = msgBean.timestamp,
            msgType = msgBean.msgType,
            body = msgBean.body
        )
    }

}