package com.tenlionsoft.aimz_k.model

import androidx.room.Dao
import androidx.room.Insert
import androidx.room.OnConflictStrategy
import androidx.room.Query
import com.tenlionsoft.aimz_k.ConvertBeanUtils

@Dao
interface MsgCategoryDao {

    /**
     * 获取全部信息
     *
     * @return
     */
    @Query("SELECT * FROM db_category")
    suspend fun getAllCategory(): List<MsgCategoryBean?>

    /**
     * 根据发送人ID获取
     */
    @Query("SELECT * FROM db_category WHERE senderId=:senderId")
    suspend fun getCategoryBySenderId(senderId: String): MsgCategoryBean?

    /**
     * 根据ID获取消息
     *
     * @param id
     * @return
     */
    @Query("SELECT * FROM db_category WHERE id =(:id) ")
    suspend fun getCategoryById(id: String): List<MsgCategoryBean>

    /**
     * 根据信息来源查询消息
     *
     * @param from
     * @return
     */
    @Query("SELECT * FROM db_category WHERE `senderId`=(:from)")
    suspend fun getCategoryByFrom(from: String?): List<MsgCategoryBean>


    /**
     * 查询与单人的聊天记录  SELECT id FROM db_msg WHERE `from`=(:form) AND `to`=(:to) OR `from`=(:to) AND `to`=(:form) ORDER BY timestamp DESC LIMIT (((:page)-1)*(:pageSize)),(:pageSize)
     */
    @Query("SELECT * FROM db_category t1 ORDER by t1.timestamp ASC LIMIT (((:page)-1)*(:pageSize)),(:pageSize)")
    suspend fun getCategoryPage(
        pageSize: Int,
        page: Int
    ): MutableList<MsgCategoryBean>


    /**
     * 清空与某个人的聊天记录
     */
    @Query("DELETE FROM db_category WHERE `senderId`=(:from) AND receiverId=(:to) OR `senderId`=(:to) AND receiverId=(:from)")
    suspend fun delChatHistory(from: String?, to: String?)

    //删除聊天
    @Query("DELETE FROM db_category WHERE senderId=:senderId")
    suspend fun delChatBySenderId(senderId: String)

    /**
     * 添加消息多个
     *
     * @param beans
     */
    @Insert(onConflict = OnConflictStrategy.REPLACE)
    suspend fun insertAll(vararg beans: MsgCategoryBean)

    /**
     * 批量添加
     */
    @Insert(onConflict = OnConflictStrategy.REPLACE)
    suspend fun insertAll(beans: List<MsgCategoryBean>)


    /**
     * 添加消息-单个
     *
     * @param bean
     */
    @Insert(onConflict = OnConflictStrategy.REPLACE)
    suspend fun insertMsg(bean: MsgCategoryBean)

    /**
     * 清除全部聊天记录
     */
    @Query("DELETE FROM db_category")
    suspend fun delAllMsg()

    /**
     * 根据发送人的ID插入或者更新
     */
    suspend fun updateOrInsert(senderId: String, msgBean: MsgBean) {
        val bean = getCategoryBySenderId(senderId)
        if (bean == null) {
            //需要转换成MsgCategoryBean
            insertMsg(ConvertBeanUtils.convertCategoryBean(msgBean))
        } else {
            updateCategory(senderId, msgBean.status, msgBean.body!!, msgBean.timestamp!!)
        }
    }

    //更新
    @Query("UPDATE db_category SET status=:status WHERE senderId=:senderId")
    suspend fun updateStatus(senderId: String, status: String)

    /**
     * 更新 消息体 时间戳 状态
     *   @Query("UPDATE db_msg SET status = :status WHERE messageId = :messageId")
     */
    @Query("UPDATE db_category SET  status= :status , body=:body , timestamp=:time WHERE senderId=:senderId")
    suspend fun updateCategory(senderId: String, status: String?, body: String, time: Long)
}