package com.tenlionsoft.aimz_k.net

import com.tenlionsoft.aimz_k.model.BaseSuccessBean
import com.tenlionsoft.aimz_k.model.FileUploadStateBean
import com.tenlionsoft.baselib.model.VersionBean
import okhttp3.MultipartBody
import okhttp3.RequestBody
import retrofit2.http.Body
import retrofit2.http.GET
import retrofit2.http.Headers
import retrofit2.http.Multipart
import retrofit2.http.POST
import retrofit2.http.Part
import retrofit2.http.Path


interface UserApi {
    /**
     * 登录
     *
     * @param user
     * @return
     */
    @Headers("Content-Type: application/json", "Accept: application/json", "projectName:usercenter")
    @POST("api/jwt/login")
    suspend fun doLogin(@Body user: RequestBody): BaseSuccessBean

    /**
     * 登陆Socket系统
     * http://192.168.0.26:8888/system/api/anonymous/login
     *
     */
    @Headers("Content-Type: application/json", "Accept: application/json", "projectName:usercenter")
    @POST("api/anonymous/login")
    suspend fun doLoginSocket(@Body user: RequestBody): BaseSuccessBean

    /**
     * 获取App版本
     *
     * @return
     */
    @Headers("token:need", "Content-Type: application/json", "Accept: application/json")
    @GET("app/appversion/get-number/{appVersionId}")
    suspend fun doCheckAppVersion(@Path("appVersionId") appId: String): VersionBean


    /**
     * 上传图片
     * image
     *
     * @param file
     * @param token
     * @return
     */
    @Headers("token:need")
    @Multipart
    @POST("api/file/upload/image")
    suspend fun doUploadImage(@Part file: MultipartBody.Part): FileUploadStateBean


    /**
     * 上传视频
     * video
     *
     * @param file
     * @param token
     * @return
     */
    @Headers("token:need")
    @Multipart
    @POST("api/file/upload/video")
    suspend fun doUploadVideo(@Part file: MultipartBody.Part): FileUploadStateBean

    /**
     * 上传文件
     */
    @Headers("token:need")
    @Multipart
    @POST("api/file/upload/file")
    suspend fun doUploadFile(@Part file: MultipartBody.Part): FileUploadStateBean

    /**
     * 上传音频
     */
    @Headers("token:need")
    @Multipart
    @POST("api/file/upload/audio")
    suspend fun doUploadAudio(@Part file: MultipartBody.Part): FileUploadStateBean
}