package com.tenlionsoft.aimz_k.page.activity

import android.annotation.SuppressLint
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.util.Log
import android.view.View
import android.view.WindowManager
import androidx.core.content.ContextCompat
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.ViewModel
import androidx.lifecycle.ViewModelProvider
import androidx.recyclerview.widget.LinearLayoutManager
import com.atwa.filepicker.core.FilePicker
import com.google.gson.Gson
import com.tenlionsoft.aimz_k.R
import com.tenlionsoft.aimz_k.databinding.ActivityChatBinding
import com.tenlionsoft.aimz_k.model.BodyContent
import com.tenlionsoft.aimz_k.model.FileDataBean
import com.tenlionsoft.aimz_k.model.MsgBean
import com.tenlionsoft.aimz_k.model.PickerType
import com.tenlionsoft.aimz_k.viewmodel.ChatPageViewModel
import com.tenlionsoft.baselib.base.BaseActivity
import com.tenlionsoft.baselib.contacts.NetConfig
import com.tenlionsoft.baselib.contacts.ProjectConfig
import com.tenlionsoft.baselib.utils.DensityUtils
import com.tenlionsoft.baselib.utils.SpUtils
import com.tenlionsoft.baselib.widget.AdapterItemClickListener
import com.tenlionsoft.baselib.widget.LoadingDialog
import com.tenlionsoft.baselib.widget.SoftKeyBoardListener
import com.tenlionsoft.baselib.widget.wheel.WheelView
import com.tenlionsoft.medialib.base.SimplePhotoActivity
import com.tenlionsoft.medialib.base.SimpleVideoActivity


/**
 * 聊天页面
 */
class ChatActivity : BaseActivity(), AdapterItemClickListener<MsgBean> {
    private lateinit var mBinding: ActivityChatBinding
    private lateinit var mLocalReceiver: LocalReceiver
    var chatPageViewModel: ChatPageViewModel? = null
    private val filePicker = FilePicker.getInstance(this)
    private var mContentHeight: Int = 0
    private var mLoading: LoadingDialog? = null;
    private var bottomHeight = -1
    override fun bindView() {
        mBinding = DataBindingUtil.setContentView(this, R.layout.activity_chat);
        window.addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS)
        window.statusBarColor =
            ContextCompat.getColor(this, com.tenlionsoft.baselib.R.color.chat_page_bg)

        val fromId = intent.getStringExtra("fromId") //传递过来的接收人
        val name = intent.getStringExtra("name")
        mBinding.tvTitle.text = if (name.isNullOrEmpty()) "临时客户" else name

        chatPageViewModel = ViewModelProvider(this, object : ViewModelProvider.Factory {
            override fun <T : ViewModel> create(modelClass: Class<T>): T {
                return ChatPageViewModel(
                    fromId!!, SpUtils.getId(), this@ChatActivity.applicationContext
                ) as T
            }
        })[ChatPageViewModel::class.java]
        chatPageViewModel!!.onItemClickListener = this
        mBinding.ivBack.setOnClickListener { finish(); }
        mBinding.viewModel = chatPageViewModel
        mBinding.lifecycleOwner = this
        //选择emoji
        mBinding.rpvEmoji.setOnEmojiPickedListener {
            mBinding.etMsg.append(it.emoji)
        }

        chatPageViewModel!!.showReplyLayout.observe(this) {
            if (it) {
                hideSoftKeyboard()//隐藏软键盘
                setContentLayoutHeight()
            } else {
                val isShow =
                    chatPageViewModel!!.showEmojiLayout.value!! || chatPageViewModel!!.showChooseLayout.value!!
                if (isShow) {
                    setContentDefaultHeight()
                }
            }
        }
        chatPageViewModel!!.showEmojiLayout.observe(this) {
            if (it) {
                hideSoftKeyboard()//隐藏软键盘
                setContentLayoutHeight()
            } else {
                val isShow =
                    chatPageViewModel!!.showReplyLayout.value!! || chatPageViewModel!!.showChooseLayout.value!!
                if (isShow) {
                    setContentDefaultHeight()
                }
            }
        }
        chatPageViewModel!!.showChooseLayout.observe(this) {
            if (it) {
                hideSoftKeyboard()//隐藏软键盘
                setContentLayoutHeight()
            } else {
                val isShow =
                    chatPageViewModel!!.showEmojiLayout.value!! || chatPageViewModel!!.showReplyLayout.value!!
                if (isShow) {
                    setContentDefaultHeight()
                }
            }
        }
        chatPageViewModel!!.showLoadDialog.observe(this) {
            if (it) {
                //显示loading
                mLoading = LoadingDialog.Builder(this).setCancelOutside(false).setCancelable(false)
                    .setMessage("上传中...").create()
                mLoading!!.show()
            } else {
                //隐藏loading
                mLoading?.dismiss()
                mLoading = null
            }
        }
        mBinding.wvView.setTextSize(14F, isSp = true)
        mBinding.wvView.setAutoFitTextSize(true)
        mBinding.wvView.setOnItemSelectedListener(object : WheelView.OnItemSelectedListener {
            override fun onItemSelected(wheelView: WheelView, data: Any, position: Int) {
                mBinding.etMsg.setText(data.toString())
            }
        })

        chatPageViewModel!!.chooseType.observe(this) {
            when (it) {
                PickerType.TYPE_PIC -> {
                    filePicker.pickMultiImage { list ->
                        if (list.isNotEmpty()) {
                            chatPageViewModel!!.uploadImages(list)
                        }
                    }
                }

                PickerType.TYPE_VIDEO -> {
                    filePicker.pickVideo { mate ->
                        if (mate != null) {
                            chatPageViewModel!!.uploadVideo(mate)
                        }
                    }
                }

                PickerType.TYPE_FILE -> {
                    filePicker.pickFile { meta ->
                        if (meta != null) {
                            chatPageViewModel!!.uploadFiles(meta)
                        }
                    }
                }

                else -> {}
            }
        }
        chatPageViewModel!!.scrollListToBottom.observe(this) {
            Log.e("TAG", "bindView:滚动  $it ")
            if (it) {
                val layoutManager = mBinding.rlvChats.layoutManager as LinearLayoutManager
                val itemCount = layoutManager.itemCount
                //滚动到底部

                mBinding.rlvChats.postDelayed({
                    layoutManager.scrollToPositionWithOffset(
                        itemCount - 1, 0
                    )
                }, 50)
                chatPageViewModel!!.scrollListToBottom.value = false
            }
        }
        mBinding.rlBottom.visibility = View.GONE
        mBinding.rlvChats.viewTreeObserver.addOnGlobalLayoutListener {
            mContentHeight = mBinding.rlvChats.height
        }

        mBinding.llBottomLayout.viewTreeObserver.addOnGlobalLayoutListener {
            if (mBinding.llBottomLayout.height == 0) {
                bottomHeight = DensityUtils.dp2px(this@ChatActivity, 300F)
            } else {
                bottomHeight = mBinding.llBottomLayout.height
            }
        }
        SoftKeyBoardListener().setChangeListener(this@ChatActivity, { h ->
            Log.e("ChatActivity", "软键盘: 显示")
            chatPageViewModel!!.showReplyLayout.value = false
            chatPageViewModel!!.showEmojiLayout.value = false
            chatPageViewModel!!.showChooseLayout.value = false
            mBinding.rlBottom.visibility = View.VISIBLE
            //重置layout高度
            val layoutParams = mBinding.rlvChats.layoutParams
            layoutParams.height = mContentHeight - h
            mBinding.rlvChats.layoutParams = layoutParams
            val layoutManager = mBinding.rlvChats.layoutManager as LinearLayoutManager
            val itemCount = layoutManager.itemCount
            //滚动到底部
            mBinding.rlvChats.postDelayed({
                layoutManager.scrollToPositionWithOffset(
                    itemCount - 1, 0
                )
            }, 100)
        }, { _ ->
            Log.e("ChatActivity", "软键盘: 隐藏")
            val isShowOther =
                chatPageViewModel!!.showReplyLayout.value!! || chatPageViewModel!!.showEmojiLayout.value!! || chatPageViewModel!!.showChooseLayout.value!!
            if (!isShowOther) {
                val layoutParams = mBinding.rlvChats.layoutParams
                layoutParams.height = mContentHeight
                mBinding.rlvChats.layoutParams = layoutParams
                mBinding.rlBottom.visibility = View.GONE
            }
        })
        registerLocalReceiver()
    }


    @SuppressLint("UnspecifiedRegisterReceiverFlag")
    private fun registerLocalReceiver() {
        mLocalReceiver = LocalReceiver()
        val intentFilter = IntentFilter()
        intentFilter.addAction(ProjectConfig.A_S_MSG_RECEIVER)
        registerReceiver(mLocalReceiver, intentFilter)
    }

    private fun unRegisterLocalReceiver() {
        unregisterReceiver(mLocalReceiver)
    }

    private fun setContentLayoutHeight() {
        mBinding.rlBottom.visibility = View.VISIBLE
        //重置layout高度
        val layoutParams = mBinding.rlvChats.layoutParams
        layoutParams.height = mContentHeight - bottomHeight
        Log.e("ChatActivity", "setContentLayoutHeight: $mContentHeight     $bottomHeight")
        mBinding.rlvChats.layoutParams = layoutParams
        val layoutManager = mBinding.rlvChats.layoutManager as LinearLayoutManager
        val itemCount = layoutManager.itemCount
        //滚动到底部
        mBinding.rlvChats.postDelayed({
            layoutManager.scrollToPositionWithOffset(
                itemCount - 1, 0
            )
        }, 100)
    }

    private fun setContentDefaultHeight() {
        val layoutParams = mBinding.rlvChats.layoutParams
        layoutParams.height = mContentHeight
        mBinding.rlvChats.layoutParams = layoutParams
        mBinding.rlBottom.visibility = View.GONE
    }

    override fun onDestroy() {
        super.onDestroy()
        unRegisterLocalReceiver()
    }

    inner class LocalReceiver : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent?) {
            when (intent?.action) {
                ProjectConfig.A_S_MSG_RECEIVER -> {
                    val msg = intent.getStringExtra("msg")
                    Log.e("LocalReceiver", "onReceive: $msg")
                    chatPageViewModel!!.receiveMsg(msg)
                }//接收到信息
            }
        }
    }

    //条目点击
    override fun onItemClick(data: MsgBean) {
        when (data.messageType) {
            ProjectConfig.MSG_TEXT -> {}
            ProjectConfig.MSG_IMG -> {
                val intent = Intent(this@ChatActivity, SimplePhotoActivity::class.java)
                val gson = Gson()
                val body = gson.fromJson(data.body, BodyContent::class.java)
                val fileDataBean = gson.fromJson(body.content, FileDataBean::class.java)
                val url = NetConfig.MAIN_URL + fileDataBean.fileUrl
                val urls = arrayListOf(url)
                intent.putStringArrayListExtra(SimplePhotoActivity.TAG_IMGURL, urls)
                startActivity(intent)
            }

            ProjectConfig.MSG_FILE -> {
                //TODO 下载文集,调用系统打开

            }

            ProjectConfig.MSG_VIDEO -> {
                val intent = Intent(this@ChatActivity, SimpleVideoActivity::class.java)
                val gson = Gson()
                val body = gson.fromJson(data.body, BodyContent::class.java)
                val fileDataBean = gson.fromJson(body.content, FileDataBean::class.java)
                intent.putExtra("url", fileDataBean.fileUrl)
                intent.putExtra("title", fileDataBean.fileName)
                startActivity(intent)
            }
        }

    }
}


