package com.tenlionsoft.aimz_k.page.fragments

import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.databinding.DataBindingUtil
import androidx.fragment.app.Fragment
import androidx.lifecycle.ViewModel
import androidx.lifecycle.ViewModelProvider
import com.bumptech.glide.Glide
import com.bumptech.glide.load.engine.DiskCacheStrategy
import com.bumptech.glide.request.RequestOptions
import com.tenlionsoft.aimz_k.R
import com.tenlionsoft.aimz_k.databinding.FragmentMineBinding
import com.tenlionsoft.aimz_k.page.activity.MainActivity
import com.tenlionsoft.aimz_k.page.activity.ManageReplyActivity
import com.tenlionsoft.aimz_k.viewmodel.MineViewModel
import com.tenlionsoft.baselib.utils.SpUtils

class MineFragment : Fragment() {
    private lateinit var mBind: FragmentMineBinding
    private lateinit var mActivity: MainActivity

    companion object {
        fun newInstance() = MineFragment()
    }

    private lateinit var viewModel: MineViewModel


    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        mBind = DataBindingUtil.inflate(
            layoutInflater,
            R.layout.fragment_mine,
            container,
            false
        )
        viewModel = ViewModelProvider(this, object : ViewModelProvider.Factory {
            override fun <T : ViewModel> create(modelClass: Class<T>): T {
                return MineViewModel() as T
            }
        })[MineViewModel::class.java]
        mBind.model = viewModel
        mBind.lifecycleOwner = this

        initView()
        return mBind.root
    }

    private fun initView() {
        mBind.tvName.text = SpUtils.getNickName()
        mBind.tvAccount.text = SpUtils.getUserName()
        val requestOptions = RequestOptions()
            .error(R.drawable.ic_user_default)
            .placeholder(R.drawable.ic_user_default)
            .skipMemoryCache(false)
            .diskCacheStrategy(DiskCacheStrategy.RESOURCE)
            .centerInside()
        Glide.with(mActivity)
            .load(SpUtils.getAvatar())
            .apply(requestOptions)
            .into(mBind.ivUserIcon)
        mBind.llSetting.setOnClickListener {
            startActivity(Intent(mActivity, ManageReplyActivity::class.java))
        }

    }


    override fun onAttach(context: Context) {
        super.onAttach(context)
        if (context is MainActivity) {
            mActivity = context
        }
    }
}