package com.tenlionsoft.aimz_k.viewmodel

import android.util.Log
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.tenlionsoft.aimz_k.adapter.ReplyAdapter
import com.tenlionsoft.aimz_k.model.DbManager
import com.tenlionsoft.aimz_k.model.ReplyBean
import com.tenlionsoft.baselib.base.BaseViewModel
import com.tenlionsoft.baselib.utils.ToastUtils
import com.tenlionsoft.baselib.widget.AdapterItemLongClickListener
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

class ManageReplyViewModel : BaseViewModel(), AdapterItemLongClickListener<ReplyBean> {

    val txtMsg = MutableLiveData("")
    val _replyList = MutableLiveData<List<ReplyBean>?>()
    var adapter: ReplyAdapter = ReplyAdapter(_replyList.value ?: emptyList(), this)
    var onItemLongClickListener: AdapterItemLongClickListener<ReplyBean> = this
    val curBean = MutableLiveData<ReplyBean?>()

    init {
        getList()
    }

    private fun getList() {
        viewModelScope.launch {
            val list: List<ReplyBean>? = withContext(Dispatchers.IO) {
                val rDao = DbManager.db.replayDao()
                val list = rDao.getAllReply()
                list
            }
            if (!list.isNullOrEmpty()) {
                _replyList.value = list
            } else {
                _replyList.value = emptyList()
            }
            Log.e("ManageReplyViewModel", "init:${adapter} ${Thread.currentThread().name}");
            adapter.setData(_replyList.value!!)
        }
    }

    fun onTxtChange(s: CharSequence, start: Int, before: Int, count: Int) {
        txtMsg.value = s.toString()
    }

    //添加短语
    fun toAddReply() {
        if (txtMsg.value != null && txtMsg.value!!.isNotEmpty()) {
            viewModelScope.launch {
                withContext(Dispatchers.IO) {
                    val rDao = DbManager.db.replayDao()
                    if (curBean.value != null) {
                        //更新
                        rDao.updateContent(curBean.value!!.id, txtMsg.value!!)
                    } else {
                        //新增
                        val b = ReplyBean(content = txtMsg.value!!)
                        rDao.insert(b)
                    }
                }
                txtMsg.value = ""
                curBean.value = null
                ToastUtils.success("保存成功")
                getList()
                showSoftKeyboard.value = false
            }
        } else {
            ToastUtils.error("请输入短语")
        }

    }

    override fun onItemLongClick(type: Int, d: ReplyBean) {
        when (type) {
            //删除
            0 -> {
                viewModelScope.launch {
                    withContext(Dispatchers.IO) {
                        val rDao = DbManager.db.replayDao()
                        rDao.delReply(d.id)
                    }
                    ToastUtils.success("删除成功")
                    getList()
                }
            }
            //修改
            1 -> {
                curBean.value = d
                txtMsg.value = d.content
                showSoftKeyboard.value = true
            }
        }
    }
}