package com.tenlionsoft.baselib.base

import android.content.Context
import android.util.Log
import android.view.MotionEvent
import android.view.View
import android.view.WindowManager
import android.view.inputmethod.InputMethodManager
import android.widget.EditText
import androidx.core.content.ContextCompat
import com.tenlionsoft.baselib.R
import kotlin.system.exitProcess


abstract class BaseActivity : DataBindingActivity() {
    //点击空白隐藏软键盘
    override fun onTouchEvent(event: MotionEvent): Boolean {
        if (event.action == MotionEvent.ACTION_DOWN) {
            Log.e("BaseActivity", "onTouchEvent: ")
            val v = currentFocus
            if (v != null && v is EditText) {
                val imm = getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
                imm.hideSoftInputFromWindow(v.windowToken, 0)
            }
        }
        return super.onTouchEvent(event)
    }

    fun exitApp() {
        android.os.Process.killProcess(android.os.Process.myPid());    //获取PID
        exitProcess(0);   //常规java、c#的标准退出法，返回值为0代表正常退出
    }

    fun hideSoftKeyboard() {
        hideSoftKeyboard(currentFocus)
    }

    private fun hideSoftKeyboard(et: View?) {
        val imm = getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
        et?.postDelayed({ imm.hideSoftInputFromWindow(et.windowToken, 0) }, 100)
    }

    fun showSoftKeyBoard(view: View?) {
        val imm = getSystemService(INPUT_METHOD_SERVICE) as InputMethodManager
        view?.postDelayed({
            imm.showSoftInput(view, InputMethodManager.SHOW_IMPLICIT)
        }, 100)
    }

    fun setStatusBarColor(dark: Boolean) {
        val window = window
        if (dark) {
            window.addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS)
            window.statusBarColor = ContextCompat.getColor(this, com.tenlionsoft.baselib.R.color.tr)
        } else {
            window.addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS)
            window.statusBarColor = ContextCompat.getColor(this, R.color.white)
        }

    }
}