package com.tenlionsoft.baselib.utils


import java.io.BufferedOutputStream
import java.io.BufferedReader
import java.io.BufferedWriter
import java.io.ByteArrayOutputStream
import java.io.File
import java.io.FileInputStream
import java.io.FileNotFoundException
import java.io.FileOutputStream
import java.io.FileWriter
import java.io.IOException
import java.io.InputStream
import java.io.InputStreamReader
import java.io.OutputStream
import java.io.RandomAccessFile
import java.io.UnsupportedEncodingException
import java.nio.ByteBuffer
import java.nio.channels.FileChannel
import java.util.List

class FileIOUtils {
    companion object{
        private
        var sBufferSize: Int = 8192
        /**
         * 将输入流写入到文件中
         *
         * @param filePath 输出文件路径.
         * @param is       输入流.
         */
        fun writeFileFromIS(filePath: String, `is`: InputStream?): Boolean {
            return writeFileFromIS(getFileByPath(filePath), `is`, false)
        }

        /**
         * 将输入流写入到文件中
         *
         * @param filePath 输出文件路径
         * @param is       输入流.
         * @param append   True 追加, false 重写.
         */
        fun writeFileFromIS(
            filePath: String,
            `is`: InputStream?,
            append: Boolean
        ): Boolean {
            return writeFileFromIS(getFileByPath(filePath), `is`, append)
        }

        /**
         * 将输入流写入到文件中
         *
         * @param file 输出的文件
         * @param is   输入流
         */
        fun writeFileFromIS(file: File?, `is`: InputStream?): Boolean {
            return writeFileFromIS(file, `is`, false)
        }

        /**
         * 将输入流写入到文件中
         *
         * @param file   输出的文件
         * @param is     输入流.
         * @param append True 添加, false 重写.
         */
        fun writeFileFromIS(
            file: File?,
            `is`: InputStream?,
            append: Boolean
        ): Boolean {
            if (!createOrExistsFile(file) || `is` == null) return false
            var os: OutputStream? = null
            try {
                os = BufferedOutputStream(FileOutputStream(file, append))
                val data = ByteArray(sBufferSize)
                var len: Int
                while ((`is`.read(data, 0, sBufferSize).also { len = it }) != -1) {
                    os!!.write(data, 0, len)
                }
                return true
            } catch (e: IOException) {
                e.printStackTrace()
                return false
            } finally {
                try {
                    `is`.close()
                } catch (e: IOException) {
                    e.printStackTrace()
                }
                try {
                    os?.close()
                } catch (e: IOException) {
                    e.printStackTrace()
                }
            }
        }


        fun writeFileFromBytesByStream(filePath: String, bytes: ByteArray?): Boolean {
            return writeFileFromBytesByStream(getFileByPath(filePath), bytes, false)
        }


        fun writeFileFromBytesByStream(
            filePath: String,
            bytes: ByteArray?,
            append: Boolean
        ): Boolean {
            return writeFileFromBytesByStream(getFileByPath(filePath), bytes, append)
        }


        fun writeFileFromBytesByStream(file: File?, bytes: ByteArray?): Boolean {
            return writeFileFromBytesByStream(file, bytes, false)
        }


        fun writeFileFromBytesByStream(
            file: File?,
            bytes: ByteArray?,
            append: Boolean
        ): Boolean {
            if (bytes == null || !createOrExistsFile(file)) return false
            var bos: BufferedOutputStream? = null
            try {
                bos = BufferedOutputStream(FileOutputStream(file, append))
                bos.write(bytes)
                return true
            } catch (e: IOException) {
                e.printStackTrace()
                return false
            } finally {
                try {
                    if (bos != null) {
                        bos.close()
                    }
                } catch (e: IOException) {
                    e.printStackTrace()
                }
            }
        }


        fun writeFileFromBytesByChannel(
            filePath: String,
            bytes: ByteArray?,
            isForce: Boolean
        ): Boolean {
            return writeFileFromBytesByChannel(getFileByPath(filePath), bytes, false, isForce)
        }

        fun writeFileFromBytesByChannel(
            filePath: String,
            bytes: ByteArray?,
            append: Boolean,
            isForce: Boolean
        ): Boolean {
            return writeFileFromBytesByChannel(getFileByPath(filePath), bytes, append, isForce)
        }

        fun writeFileFromBytesByChannel(
            file: File?,
            bytes: ByteArray?,
            isForce: Boolean
        ): Boolean {
            return writeFileFromBytesByChannel(file, bytes, false, isForce)
        }

        fun writeFileFromBytesByChannel(
            file: File?,
            bytes: ByteArray?,
            append: Boolean,
            isForce: Boolean
        ): Boolean {
            if (bytes == null) return false
            var fc: FileChannel? = null
            try {
                fc = FileOutputStream(file, append).channel
                fc.position(fc.size())
                fc.write(ByteBuffer.wrap(bytes))
                if (isForce) fc.force(true)
                return true
            } catch (e: IOException) {
                e.printStackTrace()
                return false
            } finally {
                try {
                    fc?.close()
                } catch (e: IOException) {
                    e.printStackTrace()
                }
            }
        }

        fun writeFileFromBytesByMap(
            filePath: String,
            bytes: ByteArray?,
            isForce: Boolean
        ): Boolean {
            return writeFileFromBytesByMap(filePath, bytes, false, isForce)
        }

        fun writeFileFromBytesByMap(
            filePath: String,
            bytes: ByteArray?,
            append: Boolean,
            isForce: Boolean
        ): Boolean {
            return writeFileFromBytesByMap(getFileByPath(filePath), bytes, append, isForce)
        }

        fun writeFileFromBytesByMap(
            file: File?,
            bytes: ByteArray?,
            isForce: Boolean
        ): Boolean {
            return writeFileFromBytesByMap(file, bytes, false, isForce)
        }

        fun writeFileFromBytesByMap(
            file: File?,
            bytes: ByteArray?,
            append: Boolean,
            isForce: Boolean
        ): Boolean {
            if (bytes == null || !createOrExistsFile(file)) return false
            var fc: FileChannel? = null
            try {
                fc = FileOutputStream(file, append).channel
                val mbb = fc.map(FileChannel.MapMode.READ_WRITE, fc.size(), bytes.size.toLong())
                mbb.put(bytes)
                if (isForce) mbb.force()
                return true
            } catch (e: IOException) {
                e.printStackTrace()
                return false
            } finally {
                try {
                    fc?.close()
                } catch (e: IOException) {
                    e.printStackTrace()
                }
            }
        }

        fun writeFileFromString(filePath: String, content: String?): Boolean {
            return writeFileFromString(getFileByPath(filePath), content, false)
        }

        fun writeFileFromString(
            filePath: String,
            content: String?,
            append: Boolean
        ): Boolean {
            return writeFileFromString(getFileByPath(filePath), content, append)
        }

        fun writeFileFromString(file: File?, content: String?): Boolean {
            return writeFileFromString(file, content, false)
        }

        fun writeFileFromString(
            file: File?,
            content: String?,
            append: Boolean
        ): Boolean {
            if (file == null || content == null) return false
            if (!createOrExistsFile(file)) return false
            var bw: BufferedWriter? = null
            try {
                bw = BufferedWriter(FileWriter(file, append))
                bw.write(content)
                return true
            } catch (e: IOException) {
                e.printStackTrace()
                return false
            } finally {
                try {
                    bw?.close()
                } catch (e: IOException) {
                    e.printStackTrace()
                }
            }
        }

        ///////////////////////////////////////////////////////////////////////////
        // the divide line of write and read
        ///////////////////////////////////////////////////////////////////////////
        fun readFile2List(filePath: String): MutableList<String>? {
            return readFile2List(getFileByPath(filePath), null)
        }

        fun readFile2List(filePath: String, charsetName: String?): MutableList<String>? {
            return readFile2List(getFileByPath(filePath), charsetName)
        }

        fun readFile2List(file: File?): MutableList<String>? {
            return readFile2List(file, 0, 0x7FFFFFFF, null)
        }

        fun readFile2List(file: File?, charsetName: String?): MutableList<String>? {
            return readFile2List(file, 0, 0x7FFFFFFF, charsetName)
        }


        fun readFile2List(filePath: String, st: Int, end: Int): MutableList<String>? {
            return readFile2List(getFileByPath(filePath), st, end, null)
        }


        fun readFile2List(
            filePath: String,
            st: Int,
            end: Int,
            charsetName: String?
        ): MutableList<String>? {
            return readFile2List(getFileByPath(filePath), st, end, charsetName)
        }


        fun readFile2List(file: File?, st: Int, end: Int): MutableList<String>? {
            return readFile2List(file, st, end, null)
        }


        fun readFile2List(
            file: File?,
            st: Int,
            end: Int,
            charsetName: String?
        ): MutableList<String>? {
            if (!isFileExists(file)) return null
            if (st > end) return null
            var reader: BufferedReader? = null
            try {
                var line: String
                var curLine = 1
                val list: MutableList<String> = ArrayList()
                reader = if (isSpace(charsetName)) {
                    BufferedReader(InputStreamReader(FileInputStream(file)))
                } else {
                    BufferedReader(
                        InputStreamReader(FileInputStream(file), charsetName)
                    )
                }
                while ((reader.readLine().also { line = it }) != null) {
                    if (curLine > end) break
                    if (curLine in st..end) list.add(line)
                    ++curLine
                }
                return list
            } catch (e: IOException) {
                e.printStackTrace()
                return null
            } finally {
                try {
                    reader?.close()
                } catch (e: IOException) {
                    e.printStackTrace()
                }
            }
        }


        fun readFile2String(filePath: String): String? {
            return readFile2String(getFileByPath(filePath), null)
        }

        fun readFile2String(filePath: String, charsetName: String?): String? {
            return readFile2String(getFileByPath(filePath), charsetName)
        }


        fun readFile2String(file: File?): String? {
            return readFile2String(file, null)
        }


        fun readFile2String(file: File?, charsetName: String?): String? {
            val bytes = readFile2BytesByStream(file) ?: return null
            if (isSpace(charsetName)) {
                return String(bytes)
            } else {
                try {
                    return String(bytes, charset(charsetName!!))
                } catch (e: UnsupportedEncodingException) {
                    e.printStackTrace()
                    return ""
                }
            }
        }


        fun readFile2BytesByStream(filePath: String): ByteArray? {
            return readFile2BytesByStream(getFileByPath(filePath))
        }


        fun readFile2BytesByStream(file: File?): ByteArray? {
            if (!isFileExists(file)) return null
            try {
                return is2Bytes(FileInputStream(file))
            } catch (e: FileNotFoundException) {
                e.printStackTrace()
                return null
            }
        }


        fun readFile2BytesByChannel(filePath: String): ByteArray? {
            return readFile2BytesByChannel(getFileByPath(filePath))
        }


        fun readFile2BytesByChannel(file: File?): ByteArray? {
            if (!isFileExists(file)) return null
            var fc: FileChannel? = null
            try {
                fc = RandomAccessFile(file, "r").channel
                val byteBuffer = ByteBuffer.allocate(fc.size().toInt())
                while (true) {
                    if (fc.read(byteBuffer) <= 0) break
                }
                return byteBuffer.array()
            } catch (e: IOException) {
                e.printStackTrace()
                return null
            } finally {
                try {
                    fc?.close()
                } catch (e: IOException) {
                    e.printStackTrace()
                }
            }
        }


        fun readFile2BytesByMap(filePath: String): ByteArray? {
            return readFile2BytesByMap(getFileByPath(filePath))
        }


        fun readFile2BytesByMap(file: File?): ByteArray? {
            if (!isFileExists(file)) return null
            var fc: FileChannel? = null
            try {
                fc = RandomAccessFile(file, "r").channel
                val size = fc.size().toInt()
                val mbb = fc.map(FileChannel.MapMode.READ_ONLY, 0, size.toLong()).load()
                val result = ByteArray(size)
                mbb[result, 0, size]
                return result
            } catch (e: IOException) {
                e.printStackTrace()
                return null
            } finally {
                try {
                    fc?.close()
                } catch (e: IOException) {
                    e.printStackTrace()
                }
            }
        }


        fun setBufferSize(bufferSize: Int) {
            sBufferSize = bufferSize
        }


        ///////////////////////////////////////////////////////////////////////////
        // other utils methods
        ///////////////////////////////////////////////////////////////////////////
        private fun getFileByPath(filePath: String): File? {
            return if (isSpace(filePath)) null else File(filePath)
        }

        private fun createOrExistsFile(filePath: String): Boolean {
            return createOrExistsFile(getFileByPath(filePath))
        }

        private fun createOrExistsFile(file: File?): Boolean {
            if (file == null) return false
            if (file.exists()) return file.isFile
            if (!createOrExistsDir(file.parentFile)) return false
            try {
                return file.createNewFile()
            } catch (e: IOException) {
                e.printStackTrace()
                return false
            }
        }

        private fun createOrExistsDir(file: File?): Boolean {
            return file != null && (if (file.exists()) file.isDirectory else file.mkdirs())
        }

        private fun isFileExists(file: File?): Boolean {
            return file != null && file.exists()
        }

        private fun isSpace(s: String?): Boolean {
            if (s == null) return true
            var i = 0
            val len = s.length
            while (i < len) {
                if (!Character.isWhitespace(s[i])) {
                    return false
                }
                ++i
            }
            return true
        }

        private fun is2Bytes(`is`: InputStream?): ByteArray? {
            if (`is` == null) return null
            var os: ByteArrayOutputStream? = null
            try {
                os = ByteArrayOutputStream()
                val b = ByteArray(sBufferSize)
                var len: Int
                while ((`is`.read(b, 0, sBufferSize).also { len = it }) != -1) {
                    os.write(b, 0, len)
                }
                return os.toByteArray()
            } catch (e: IOException) {
                e.printStackTrace()
                return null
            } finally {
                try {
                    `is`.close()
                } catch (e: IOException) {
                    e.printStackTrace()
                }
                try {
                    os?.close()
                } catch (e: IOException) {
                    e.printStackTrace()
                }
            }
        }
    }
}