package com.tenlionsoft.baselib.utils

import android.content.ActivityNotFoundException
import android.content.Context
import android.content.Intent
import android.net.Uri
import android.os.Build
import android.util.Log
import android.widget.Toast
import androidx.core.content.FileProvider
import java.io.File
import java.util.Locale

class ProiderUtils {
    companion object {

        fun getFileProviderName(context: Context): String {
            return context.packageName + ".provider"
        }

        fun openFile(context: Context, file: File) {
            try {
                val intent = Intent()
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                // 设置intent的Action属性
                intent.setAction(Intent.ACTION_VIEW)
                // 获取文件file的MIME类型
                val type = getMIMEType(file)
                // 设置intent的data和Type属性。
                val uri = if (Build.VERSION.SDK_INT <= Build.VERSION_CODES.M) {
                    Uri.fromFile(file)
                } else {
                    FileProvider.getUriForFile(context, getFileProviderName(context), file)
                }
                intent.setDataAndType( /* uri */uri, type)
                // 跳转
                context.databaseList()
                context.startActivity(intent)

                // Intent.createChooser(intent, "请选择对应的软件打开该附件！");
            } catch (e: ActivityNotFoundException) {
                Toast.makeText(context, "sorry附件不能打开，请下载相关软件！", Toast.LENGTH_SHORT)
                    .show()
            }
        }

        private fun getMIMEType(file: File): String {
            var type = "*/*"
            val fName = file.name
            // 获取后缀名前的分隔符"."在fName中的位置。
            val dotIndex = fName.lastIndexOf(".")
            if (dotIndex < 0) {
                return type
            }
            /* 获取文件的后缀名 */
            val end = fName.substring(dotIndex, fName.length).lowercase(Locale.getDefault())
            if (end === "") return type
            // 在MIME和文件类型的匹配表中找到对应的MIME类型。
            for (i in MIME_MapTable.indices) {
                if (end == MIME_MapTable[i][0]) type = MIME_MapTable[i][1]
            }
            return type
        }

        /*
     * 传文件路径打开文件
     *
     */
        fun openFile(context: Context, filePath: String) {
            Log.e("CallOtherOpenFile", "openFile: $filePath")
            val file = File(filePath)
            openFile(context, file)
        }

        // 可以自己随意添加
        var MIME_MapTable: Array<Array<String>> = arrayOf(
            // {后缀名，MIME类型}
            arrayOf(".3gp", "video/3gpp"),
            arrayOf(".apk", "application/vnd.android.package-archive"),
            arrayOf(".asf", "video/x-ms-asf"),
            arrayOf(".avi", "video/x-msvideo"),
            arrayOf(".bin", "application/octet-stream"),
            arrayOf(".bmp", "image/bmp"),
            arrayOf(".c", "text/plain"),
            arrayOf(".class", "application/octet-stream"),
            arrayOf(".conf", "text/plain"),
            arrayOf(".cpp", "text/plain"),
            arrayOf(".doc", "application/msword"),
            arrayOf(
                ".docx",
                "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
            ),
            arrayOf(".xls", "application/vnd.ms-excel"),
            arrayOf(
                ".xlsx",
                "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
            ),
            arrayOf(".exe", "application/octet-stream"),
            arrayOf(".gif", "image/gif"),
            arrayOf(".gtar", "application/x-gtar"),
            arrayOf(".gz", "application/x-gzip"),
            arrayOf(".h", "text/plain"),
            arrayOf(".htm", "text/html"),
            arrayOf(".html", "text/html"),
            arrayOf(".jar", "application/java-archive"),
            arrayOf(".java", "text/plain"),
            arrayOf(".jpeg", "image/jpeg"),
            arrayOf(".jpg", "image/jpeg"),
            arrayOf(".js", "application/x-javascript"),
            arrayOf(".log", "text/plain"),
            arrayOf(".m3u", "audio/x-mpegurl"),
            arrayOf(".m4a", "audio/mp4a-latm"),
            arrayOf(".m4b", "audio/mp4a-latm"),
            arrayOf(".m4p", "audio/mp4a-latm"),
            arrayOf(".m4u", "video/vnd.mpegurl"),
            arrayOf(".m4v", "video/x-m4v"),
            arrayOf(".mov", "video/quicktime"),
            arrayOf(".mp2", "audio/x-mpeg"),
            arrayOf(".mp3", "audio/x-mpeg"),
            arrayOf(".mp4", "video/mp4"),
            arrayOf(".mpc", "application/vnd.mpohun.certificate"),
            arrayOf(".mpe", "video/mpeg"),
            arrayOf(".mpeg", "video/mpeg"),
            arrayOf(".mpg", "video/mpeg"),
            arrayOf(".mpg4", "video/mp4"),
            arrayOf(".mpga", "audio/mpeg"),
            arrayOf(".msg", "application/vnd.ms-outlook"),
            arrayOf(".ogg", "audio/ogg"),
            arrayOf(".pdf", "application/pdf"),
            arrayOf(".png", "image/png"),
            arrayOf(".pps", "application/vnd.ms-powerpoint"),
            arrayOf(".ppt", "application/vnd.ms-powerpoint"),
            arrayOf(
                ".pptx",
                "application/vnd.openxmlformats-officedocument.presentationml.presentation"
            ),
            arrayOf(".prop", "text/plain"), arrayOf(".rc", "text/plain"),
            arrayOf(".rmvb", "audio/x-pn-realaudio"), arrayOf(".rtf", "application/rtf"),
            arrayOf(".sh", "text/plain"), arrayOf(".tar", "application/x-tar"),
            arrayOf(".tgz", "application/x-compressed"), arrayOf(".txt", "text/plain"),
            arrayOf(".wav", "audio/x-wav"), arrayOf(".wma", "audio/x-ms-wma"),
            arrayOf(".wmv", "audio/x-ms-wmv"),
            arrayOf(".wps", "application/vnd.ms-works"), arrayOf(".xml", "text/plain"),
            arrayOf(".z", "application/x-compress"),
            arrayOf(".zip", "application/x-zip-compressed"),
            arrayOf(".tif", "image/tiff"),
            arrayOf(".tiff", "image/tiff"),
            arrayOf(".amr", "video/*"),  //            { "", "*/*" }
        )
    }
}