package com.tenlionsoft.baselib.widget.chat

import android.annotation.SuppressLint
import android.content.Context
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.Paint
import android.graphics.RectF
import android.os.Handler
import android.os.Looper
import android.os.Message
import android.util.AttributeSet
import android.widget.LinearLayout
import com.tenlionsoft.baselib.R
import com.tenlionsoft.baselib.utils.DensityUtils

class IndicatorView : LinearLayout {
    private var indicatorColor = Color.rgb(0, 0, 0)
    private var indicatorColorSelected = Color.rgb(0, 0, 0)
    private var indicatorWidth = 0
    private var gravity = 0

    private var indicatorCount = 0
    private var currentIndicator = 0

    private val handler: Handler = object : Handler(Looper.getMainLooper()) {
        override fun handleMessage(msg: Message) {
            if (msg.what == 0x12) {
                invalidate()
            }
        }
    }

    constructor(context: Context) : super(context) {
    }

    constructor(context: Context, attrs: AttributeSet?) : super(context, attrs) {
        if (attrs != null) {
            val typedArray = getContext().obtainStyledAttributes(attrs, R.styleable.IndicatorView)
            indicatorColor =
                typedArray.getColor(R.styleable.IndicatorView_indicatorColor, Color.rgb(0, 0, 0))
            indicatorColorSelected = typedArray.getColor(
                R.styleable.IndicatorView_indicatorColorSelected,
                Color.rgb(0, 0, 0)
            )
            indicatorWidth = DensityUtils.dp2px(
                context,
                typedArray.getFloat(R.styleable.IndicatorView_indicatorWidth, 0F)
            )
            gravity = typedArray.getInt(R.styleable.IndicatorView_gravity, 0)
            typedArray.recycle()
        }
    }


    override fun onDraw(canvas: Canvas) {
        val viewWidth = width
        val viewHeight = height
        val totalWidth = indicatorWidth * (2 * indicatorCount - 1)

        val paint = Paint()
        paint.isAntiAlias = true

        if (indicatorCount > 0) {
            for (i in 0 until indicatorCount) {
                if (i == currentIndicator) {
                    paint.color = indicatorColorSelected
                } else {
                    paint.color = indicatorColor
                }
                var left = (viewWidth - totalWidth) / 2 + (i * 2 * indicatorWidth)
                when (gravity) {
                    0 -> left = (viewWidth - totalWidth) / 2 + (i * 2 * indicatorWidth)
                    1 -> left = i * 2 * indicatorWidth
                    2 -> left = viewWidth - totalWidth + (i * 2 * indicatorWidth)
                }
                val top = (viewHeight - indicatorWidth) / 2
                val right = left + indicatorWidth
                val bottom = top + indicatorWidth
                val rectF = RectF(left.toFloat(), top.toFloat(), right.toFloat(), bottom.toFloat())
                canvas.drawOval(rectF, paint)
            }
        }
    }

    fun setIndicatorCount(indicatorCount: Int) {
        this.indicatorCount = indicatorCount
    }

    fun setCurrentIndicator(currentIndicator: Int) {
        this.currentIndicator = currentIndicator
        handler.sendEmptyMessage(0x12)
    }
}